@@ -4,7 +4,7 @@
 #         Edward Kawas
 # For copyright and disclaimer see below.
 #
-# $Id: Core.pm,v 1.19 2010-03-09 17:36:14 ubuntu Exp $
+# $Id: Core.pm,v 1.5 2009-09-17 16:30:59 ubuntu Exp $
 #-----------------------------------------------------------------
 package SADI::RDF::Core;
 use strict;
@@ -12,7 +12,6 @@ use strict;
 use Carp;
 
 use Template;
- 
 use FindBin qw( $Bin );
 use lib $Bin;
 use File::Spec;
@@ -31,11 +30,11 @@ use base ("SADI::Base");
 
 # add versioning to this module
 use vars qw /$VERSION/;
-$VERSION = sprintf "%d.%02d", q$Revision: 1.19 $ =~ /: (\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.5 $ =~ /: (\d+)\.(\d+)/;
 
 =head1 NAME
 
-SADI::RDF::Core - A Perl package for SADI services
+     SADI::RDF::Core -- A Perl package for SADI services
 
 =head1 DESCRIPTION
 
@@ -224,7 +223,7 @@ sub getInputNodes {
 	  RDF::Core::Resource->new("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
 	my $inputtype = RDF::Core::Resource->new($predicate);
 
-#  we need the input types to be "Input" because a client can honestly send us a more complex type that inherits and it wont be understood without a reasoner
+#  we need the input types to be "Input" because a client can honestly send us a more complex type that inherits and it wpont be understood without a reasoner
 
 	my $yesno = $model->existsStmt( undef, $type, $inputtype );
 	return () unless $yesno;
@@ -292,9 +291,9 @@ sub getLiteralPropertyValues {
       subject   => the URI of the subject for which you want to retrieve statements for
       object    => the URI of the object for which you want to retrieve statements for
       predicate => the URI of the predicate for which you want to retrieve statements for
-
+  
   B<subject, object and predicate are all optional.>
-
+  
   returns
       a reference to an array of RDF::Core::Statements that match the given subject, object and predicate
 
@@ -352,9 +351,9 @@ sub getStatements {
   %args
       subject   => the URI of the subject for which you want to retrieve objects for
       predicate => the URI of the predicate for which you want to retrieve objects for
-
+  
   B<subject, object and predicate are all optional.>
-
+  
   returns
       a reference to an array of RDF::Core::Resource that match the given subject and predicate
 
@@ -393,11 +392,11 @@ sub getObjects {
 
   add an output triple to the model; the predicate of the triple
   is automatically extracted from the ServicePredicate.
-
+  
   You can pass a URI or an RDF::Core::Resource as the "value" argument.  
   The node is automatically rdf:typed as the OutputClass if you include
   the "typed_as_output" argument as true.
-
+  
   If you pass a "value" that looks like a URI, then this routine WILL ASSUME
   THAT YOU WANT IT TO BE AN OBJECT, NOT A SCALAR VALUE.  To over-ride this,
   set the boolean "force_literal" argument.  If you pass an RDF::Core::Resource
@@ -405,19 +404,12 @@ sub getObjects {
   will be extracted and added as a literal value rather than as an object.
 
   args
-
-     node => $URI  (the URI string, RDF::Core::Resource of the subject node or 
-             a SADI::Data::OWL::Class (object generated using sadi-generate-datatypes)).
-             In the event of an OWL class, all other args are ignored.
-
+     node => $URI  (the URI string or RDF::Core::Resource of the subject node)
      value => $val  (a string value)
-
-     predicate => $URI (required unless node isa SADI::Data::OWL::Class- the predicate to put between them.)
-
+     predicate => $URI (optional - the predicate to put between them.  
+                        Defaults to $self->ServicePredicate)
      typed_as_output => boolean (if present output is rdf:typed as output class)
-
      force_literal => boolean
-
      label => $label (string); label for value node, only if value is a URI
 
 =cut
@@ -428,13 +420,7 @@ sub addOutputData {
 	my $subject     = $args{node};
 	if ( ref($subject) =~ /RDF::Core::Resource/ ) {
 		$subject = RDF::Core::Resource->new( $subject->getURI );
-	} elsif ( UNIVERSAL::isa($subject, 'SADI::Data::OWL::Class') ) {
-		# using generated modules, so get their statements and return
-		foreach ( @{ $subject->_get_statements } ) {
-            $self->_addToModel( statement => $_ );
-        }
-        return;
-    } else {
+	} else {
 		$subject = RDF::Core::Resource->new($subject);
 	}
 	my $object         = $args{value};
@@ -452,50 +438,46 @@ sub addOutputData {
 	my $predicate =
 	  $predicate_sent
 	  ? RDF::Core::Resource->new($predicate_sent)
-	  : undef;
-	  #: RDF::Core::Resource->new( $self->Signature->ServicePredicate );
-	$LOG->warn("Cannot completely addOutputData() without a predicate!\nPlease check how you are calling addOutputData() and include a predicate!")
-	  unless defined $predicate;
-	if (defined $predicate) {
-		if ( ref($object) && ( ref($object) =~ /RDF::Core/ ) )
-		{        # did they send us an objectt of the right type?
-			if ($force_literal)
-			{ # did they want the URI of that object as a literal value (very rare, but why not)
-				my $URI = $object->getURI;
-				$object = RDF::Core::Literal->new($URI);
-	
-				my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
-				$self->_addToModel( statement => $statement );
-			} else {    # they sent an RDF::Core node that we should simply add to the graph
-				my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
-				$self->_addToModel( statement => $statement );
-				if ($label) {
-					$label = RDF::Core::Literal->new($label);
-					my $lab = RDF::Core::Resource->new(
-											  'http://www.w3.org/2000/01/rdf-schema#label');
-					$statement = RDF::Core::Statement->new( $object, $lab, $label );
-					$self->_addToModel( statement => $statement );
-				}
-			}
-		} else {    # they sent a literal value... is it a URI-type thing?
-			if ( $object =~ /\S+\:\S+\.\S+/ && !$force_literal )
-			{ # a terrible regexp for a URI... should find the one that is sanctioned by the W3C URI RFC... look for it later...
-				$object = RDF::Core::Resource->new($object);
-				my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
-				$self->_addToModel( statement => $statement );
-				if ($label) {
-					$label = RDF::Core::Literal->new($label);
-					my $lab = RDF::Core::Resource->new(
-											  'http://www.w3.org/2000/01/rdf-schema#label');
-					$statement = RDF::Core::Statement->new( $object, $lab, $label );
-					$self->_addToModel( statement => $statement );
-				}
-			} else {
-				$object = RDF::Core::Literal->new($object);
-				my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
+	  : RDF::Core::Resource->new( $self->Signature->ServicePredicate );
+
+	if ( ref($object) && ( ref($object) =~ /RDF::Core/ ) )
+	{        # did they send us an objectt of the right type?
+		if ($force_literal)
+		{ # did they want the URI of that object as a literal value (very rare, but why not)
+			my $URI = $object->getURI;
+			$object = RDF::Core::Literal->new($URI);
+
+			my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
+			$self->_addToModel( statement => $statement );
+		} else {    # they sent an RDF::Core node that we should simply add to the graph
+			my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
+			$self->_addToModel( statement => $statement );
+			if ($label) {
+				$label = RDF::Core::Literal->new($label);
+				my $lab = RDF::Core::Resource->new(
+										  'http://www.w3.org/2000/01/rdf-schema#label');
+				$statement = RDF::Core::Statement->new( $object, $lab, $label );
 				$self->_addToModel( statement => $statement );
 			}
 		}
+	} else {    # they sent a literal value... is it a URI-type thing?
+		if ( $object =~ /\S+\:\S+\.\S+/ && !$force_literal )
+		{ # a terrible regexp for a URI... should find the one that is sanctioned by the W3C URI RFC... look for it later...
+			$object = RDF::Core::Resource->new($object);
+			my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
+			$self->_addToModel( statement => $statement );
+			if ($label) {
+				$label = RDF::Core::Literal->new($label);
+				my $lab = RDF::Core::Resource->new(
+										  'http://www.w3.org/2000/01/rdf-schema#label');
+				$statement = RDF::Core::Statement->new( $object, $lab, $label );
+				$self->_addToModel( statement => $statement );
+			}
+		} else {
+			$object = RDF::Core::Literal->new($object);
+			my $statement = RDF::Core::Statement->new( $subject, $predicate, $object );
+			$self->_addToModel( statement => $statement );
+		}
 	}
 	if ($add_type_data) {
 		my $output_type = RDF::Core::Resource->new( $self->Signature->OutputClass );
@@ -559,7 +541,7 @@ sub serializeOutputModel {
   according to the SADI best-practices, the service URL should return the
   interface document if you call it with GET.  Here we auto-generate that
   document.
-
+ 
   $service->getServiceInterface()
 
 =cut
@@ -580,14 +562,10 @@ sub getServiceInterface {
 	my $authoritative = $self->Signature->Authoritative();
 	my $authority     = $self->Signature->Authority() ;
 	my $sigURL        = $self->Signature->SignatureURL() || "";
-	my @tests         = $self->Signature->UnitTest || ();
 
 	# generate from template
-	my $sadi_interface_signature= '';
-	my $tt = Template->new( 
-	   ABSOLUTE => 1, 
-	   TRIM => 1, 
-	);
+	my $sadi_interface_signature;
+	my $tt = Template->new( ABSOLUTE => 1, TRIM => 1 );
 	my $input = File::Spec->rel2abs(
 					  SADI::Utils->find_file(
 						  $Bin, 'SADI', 'Generators', 'templates', 'service-signature.tt'
@@ -610,67 +588,13 @@ sub getServiceInterface {
 					 authoritative => $authoritative,
 					 authority     => $authority,
 					 sigURL        => $sigURL,
-					 tests         => @tests,
 				  },
-				  \$sadi_interface_signature
+				  $sadi_interface_signature
 	) || $LOG->logdie( $tt->error() );
 
 	return $sadi_interface_signature;
 }
 
-
-sub _add_error {
-    my ($self, $msg, $comment, $stack) = @_;
-
-    # generate from template
-    my $error_rdf = '';
-    my $tt = Template->new( ABSOLUTE => 1, TRIM => 1 );
-    my $input = File::Spec->rel2abs(
-                      SADI::Utils->find_file(
-                          $Bin, 'SADI', 'Generators', 'templates', 'service-error.tt'
-                      )
-    );
-    $msg ||= '';
-    $comment ||= '';
-    $stack ||= '';
-    
-    use CGI;
-    $tt->process(
-                  $input,
-                  {
-                     message  => CGI::escapeHTML($msg),
-                     comment  => CGI::escapeHTML($comment),
-                     stack    => CGI::escapeHTML($stack),
-                  },
-                  \$error_rdf
-    ) || $LOG->logdie( $tt->error() );
-    # if problem generating error doc, return
-    return unless defined ($error_rdf);
-    return if $error_rdf eq '';
-
-    # parse the error doc now
-	my $storage = new RDF::Core::Storage::Memory;
-	my $model = new RDF::Core::Model( Storage => $storage );
-	my %options = (
-	                Model      => $model,
-	                Source     => $error_rdf,
-	                SourceType => 'string',
-	);
-	my $parser = new RDF::Core::Model::Parser(%options);
-	$parser->parse;
-	my $enumerator = $model->getStmts;
-	my $statement  = $enumerator->getFirst;
-	# add statement to our output model
-	while ( defined $statement ) {
-	    $self->_addToModel(statement=>$statement);
-	    $statement = $enumerator->getNext;
-	}
-	$enumerator->close;
-	# done;
-	return;
-}
-
-
 sub _prepareOutputModel {
 	my ($self) = @_;
 	my $storage = new RDF::Core::Storage::Memory;
